﻿//
// Copyright (c) 2017, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Script.Serialization;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing.Drawing2D;

using MindFusion.Charting;
using MindFusion.Charting.Components;
using MindFusion.Charting.Gauges;
using MindFusion.Charting.WebForms;

using System.Drawing;
using Brush = MindFusion.Drawing.Brush;
using SolidBrush = MindFusion.Drawing.SolidBrush;
using LinearGradientBrush = MindFusion.Drawing.LinearGradientBrush;


namespace Test
{
	public partial class CustomData : System.Web.UI.Page
	{
		protected void Page_Init(object sender, EventArgs e)
		{
			chart.RegisterCustomType(typeof(PatientSeries), "PatientSeries");

			// register converters for serializing .NET items into JSON objects
			chart.RegisterConverters(new JavaScriptConverter[] { new PatientSeriesConverter() });
		}

		protected void Page_Load(object sender, EventArgs e)
		{
			if (!IsPostBack)
			{
				var collection = new ObservableCollection<Series>();
				for (var i = 0; i < 3; i++)
				{
					if (i == 0)
					{
						collection.Add(new PatientSeries(
							new List<Patient>(){
							new Patient("Adams", 15, 70, "BMI > 25"),
							new Patient("Byrnes", 4, 20, "BMI > 25"),
							new Patient("Clyde", 25, 95, "BMI > 25"),
							new Patient("Davis", 32, 108, "BMI > 25"),
							new Patient("Evans", 51, 83, "BMI > 25")}));
					}
					if (i == 1)
					{
						collection.Add(new PatientSeries(
							new List<Patient>() {
							new Patient("Fox", 35, 180, "BMI > 30"),
							new Patient("Gene", 14, 77, "BMI > 30"),
							new Patient("Harold", 7, 65, "BMI > 30"),
							new Patient("Ian", 48, 125, "BMI > 30"),
							new Patient("James", 67, 98, "BMI > 30")}));
					}
				}

				chart.Series = collection;
				chart.Title = "Patient BM Index by Age";
				chart.BackColor = ColorTranslator.FromHtml("#c0c0c0");

				chart.XAxis.MinValue = 0;
				chart.XAxis.MaxValue = 80;
				chart.XAxis.Title = "Age";
				chart.YAxis.MinValue = 0;
				chart.YAxis.MaxValue = 250;
				chart.YAxis.Title = "Weight";

				chart.Theme.CommonSeriesFills = chart.Theme.CommonSeriesStrokes =
					new List<Brush>(){
						new SolidBrush(Color.FromArgb(178, 206, 0, 0)), 
						new SolidBrush(Color.FromArgb(178, 45, 57, 86))};

				chart.LegendHorizontalAlignment = LayoutAlignment.Far;
				chart.Theme.LegendBackground = new SolidBrush(ColorTranslator.FromHtml("#9caac6"));
				chart.Theme.HighlightStroke = new SolidBrush(ColorTranslator.FromHtml("#c0c0c0"));
				chart.LegendTitle = "";
				chart.AllowMoveLegend = false;
				chart.GridType = GridType.Horizontal;
				chart.Theme.GridColor1 = chart.Theme.GridColor2 = Color.FromArgb(224, 233, 233);
				chart.Theme.GridLineStyle = DashStyle.Dash;

				chart.Theme.TitleFontSize = 20;
				chart.Theme.TitleFontStyle = FontStyle.Underline;

			}
		}


		public class Patient
		{
			public Patient(string name, double age, double weight, string bmindex)
			{
				Name = name;
				Age = age;
				Weight = weight;
				Bmindex = bmindex;
			}

			public string Name;
			public double Age;
			public double Weight;
			public string Bmindex;
		}

		public class PatientSeries : Series
		{
			public PatientSeries(List<Patient> patients)
			{
				Values = patients;
			}

			public double GetValue(int index, int dimension)
			{
				if (dimension == 0)
					return Values[index].Age;
				else return Values[index].Weight;
			}

			public string GetLabel(int index, LabelKinds kind)
			{
				if (kind == LabelKinds.InnerLabel) return Values[index].Name;
				if (kind == LabelKinds.ToolTip) return "${this.values[index].name}(${this.values[index].bmindex})";

				return null;
			}

			public event EventHandler DataChanged;

			public int Dimensions
			{
				get { return 3; }
			}

			public bool IsEmphasized(int index)
			{
				return false;
			}

			public bool IsSorted(int dimension)
			{
				return false;
			}

			public int Size
			{
				get { return Values.Count; }
			}

			public LabelKinds SupportedLabels
			{
				get { return LabelKinds.InnerLabel | LabelKinds.ToolTip; }
			}

			public string Title
			{
				get { return Values[0].Bmindex; }
			}


			public List<Patient> Values;
		}

		/// <summary>
		/// Serializes PatientSeries instances.
		/// </summary>
		public class PatientSeriesConverter : JavaScriptConverter
		{

			/// <summary>
			/// SupportedTypes override.
			/// </summary>
			public override IEnumerable<Type> SupportedTypes
			{
				get
				{
					return new List<Type>(new Type[] { typeof(PatientSeries) });
				}
			}

			/// <summary>
			/// Deserialize override.
			/// </summary>
			public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
			{
				var patients = new List<Patient>();

				var values = (ArrayList)dictionary["values"];
				for (var i = 0; i < values.Count; i++)
				{
					var data = (IDictionary<string, object>)values[i];
					var p = new Patient(data["name"].ToString(), double.Parse(data["age"].ToString()),
						double.Parse(data["weight"].ToString()), data["bmindex"].ToString());
					patients.Add(p);
				}

				PatientSeries series = new PatientSeries(patients);
				return series;
			}

			/// <summary>
			/// Serialize override.
			/// </summary>
			public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
			{
				IDictionary<string, object> json = new Dictionary<string, object>();

				PatientSeries series = (PatientSeries)obj;

				json.Add("values", series.Values);

				return json;
			}

		}

		protected void Button1_Click(object sender, EventArgs e)
		{

		}
	}
}